#include <stdio.h>
#include <unistd.h>
#include <string>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <linux/ioctl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>

#include "ecs_driverlib.h"
#include "pcie40_driverlib.h"
#include "pcie40_ioctl.h"

//`p40_ecs_open`
int p40_ecs_open(int dev, int bar, uint32_t **regs)
{
  int fd;
  char devname[32];

  std::string suffix;
  switch (bar) {
    case 0:
      suffix = "bar0";
      break;
    case 2:
      suffix = "bar2";
      break;
    default:
      return -EINVAL;
  }

  if (p40_get_devname(dev, suffix.c_str(), devname, sizeof(devname)) < 0) {
    perror("p40_get_devname()");
    return -1;
  }

  if ((fd = open(devname, O_RDWR)) == -1){
    perror("open()");
    return -1;
  }

  int bar_size = -1 ; 
  bar_size = p40_ecs_bar_size( fd ) ;

  if (bar_size < 0) {
    close(fd);
    return -1;
  }

  *regs = (uint32_t *)mmap(NULL, bar_size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
  if (*regs == NULL) {
    perror("mmap()");
    close(fd);
    return -1;
  }

  return fd;
}

//`p40_ecs_close`
void p40_ecs_close(int fd, uint32_t *regs, int bar)
{
  int bar_size = -1 ;
  bar_size = p40_ecs_bar_size(fd);
  munmap(regs, bar_size);
  close(fd);
}

//`p40_ecs_bar_size`
int p40_ecs_bar_size(int fd)
{
  int ret;

  //? See ioctl.pcie`P40_ECS_GET_BAR_SIZE` .
  ret = ioctl(fd, P40_ECS_GET_BAR_SIZE);
  if (ret < 0) {
    perror("ioctl(P40_ECS_GET_BAR_SIZE)");
    return -1;
  }
  return ret;
}

//`p40_ecs_w32`
void p40_ecs_w32(uint32_t volatile *regs, uint32_t addr, uint32_t val)
{
  regs[addr >> 2] = val;
}

//`p40_ecs_r32`
uint32_t p40_ecs_r32(uint32_t volatile *regs, uint32_t addr)
{
  return regs[addr >> 2];
}

int pcie40_ecs_slc_lock( int fd , int ch ) {
  return ioctl(fd, P40_ECS_SLC_LOCK, ch);
}
int pcie40_ecs_slc_unlock( int fd , int ch ) {
  return ioctl(fd, P40_ECS_SLC_UNLOCK, ch);
}
int pcie40_ecs_slc_checklock( int fd , int ch ) {
  return ioctl(fd, P40_ECS_SLC_CHECKLOCK, ch);
}